/*!
* @UTILITY FUNCTIONS
* Visit http://lofgames.com/ 
*
* Creative Common Licence
*
* Distributed under the terms of the MIT license.
* http://www.opensource.org/licenses/mit-license.html
*
* This notice shall be included in all copies or substantial portions of the Software.
*/
var sfx_arr = [];
var music_arr = [];
function playSFX(sfx_name,inVol = 1) {
	var sInstance = createjs.Sound.play(sfx_name);
	sfx_arr.push(sInstance);
	sInstance.sound_id = sfx_name;
	sInstance.inVol = inVol; 
	sInstance.volume = gamevars(0).sobj.sfxVolume*sInstance.inVol;
	sInstance.addEventListener("complete", onSoundComplete);
}

function onSoundComplete(event) {
	var sInstance = event.target;
	sInstance.removeEventListener("complete", onSoundComplete);
	sfx_arr.splice(sfx_arr.indexOf(sInstance), 1);
}
function playMusic(music_name,inVol=1) {
	var mInstance = createjs.Sound.play(music_name, "none", 0, 0, -1);
	music_arr.push(mInstance);
	mInstance.inVol = inVol;
	mInstance.music_id = music_name;
	mInstance.volume = gamevars(0).sobj.musicVolume*mInstance.inVol;
}
function removeAllSounds(){
	while(sfx_arr.length > 0){
		var sInstance = sfx_arr[0];
		removeSound(sInstance.sound_id);
	}
}
function removeSound(sfx_name) {
	for (var sInstance of sfx_arr) {
		if (sInstance.sound_id == sfx_name) {
			sInstance.stop();
			sInstance.removeEventListener("complete", onSoundComplete);
			sfx_arr.splice(sfx_arr.indexOf(sInstance), 1);
			sInstance.destroy();
		}
	}
}
function removeMusic(music_name) {
	for (var mInstance of music_arr) {
		if (mInstance.music_id == music_name) {
			mInstance.stop();
			music_arr.splice(music_arr.indexOf(mInstance), 1);
			mInstance.destroy();
		}
	}
}
function updateMusicVolume() {
	for (var mInstance of music_arr) {
		mInstance.volume = gamevars(0).sobj.musicVolume*mInstance.inVol;
	}
}
function updateSfxVolume() {
	for (var sInstance of sfx_arr) {
		sInstance.volume = gamevars(0).sobj.sfxVolume*sInstance.inVol;
	}
}
function puaseSoundMusic(){
	for (var mInstance of music_arr) {
		mInstance.volume = 0;
	}
	for (var sInstance of sfx_arr) {
		sInstance.volume = 0;
	}
}
function resumeSoundMusic(){
	updateMusicVolume();
	updateSfxVolume();
}

function toggleFSButton() {
	var doc = window.document;
	var docEl = doc.documentElement;

	var requestFullScreen = docEl.requestFullscreen || docEl.mozRequestFullScreen || docEl.webkitRequestFullScreen || docEl.msRequestFullscreen;
	var cancelFullScreen = doc.exitFullscreen || doc.mozCancelFullScreen || doc.webkitExitFullscreen || doc.msExitFullscreen;

	if (!doc.fullscreenElement && !doc.mozFullScreenElement && !doc.webkitFullscreenElement && !doc.msFullscreenElement) {
		requestFullScreen.call(docEl);
	} else {
		cancelFullScreen.call(doc);
	}
}

function randomizeArray(arrayx) {
	var newArray = new Array();

	while (arrayx.length > 0) {
		var idx = Math.floor(Math.random() * arrayx.length);
		newArray.push(arrayx[idx]);
		arrayx.splice(idx, 1);
	}

	for (var i = 0; i < newArray.length; i++) {
		arrayx.push(newArray[i]);
	}

}

function sortOn(arrarg, prop_list, acdclist) {
	arrarg.sort(function (a, b) {
		for (var i = 0; i < prop_list.length; i++) {
			var prop = prop_list[i];
			var dec = acdclist[i];
			if (a[prop] != b[prop]) {
				return (dec ? b[prop] - a[prop] : a[prop] - b[prop]);
			}
		}
	});
}
function sortNumArray(arrarg, decending = false) {
	arrarg.sort(function (a, b) {
		if (decending) {
			return b - a;
		} else {
			return a - b;
		}
	});
}
function getTimer() {
	return Number(new Date());
}
function getTimeString(seconds) {
	var min = Math.floor(seconds / 60);
	var sec = Math.floor(seconds % 60);

	var time_str = "0" + min + ":";
	if (min > 9) {
		time_str = min + ":";
	} else {
		time_str = "0" + min + ":";
	}
	if (sec < 10) {
		time_str += "0" + sec;
	} else {
		time_str += "" + sec;
	}
	return time_str;

}
function stringToNumArray(arrayAsString) {
	var arr = arrayAsString.split(",");
	for (var i = 0; i < arr.length; i++) {
		arr[i] = Number(arr[i]);
	}
	return arr;
}
function createClone(mc) {
	var newMC = new createjs.MovieClip();
	for (var i = 0; i < mc.numChildren; i++) {
		var obj = mc.getChildAt(i);
		var copyobj = null;
		if (obj instanceof createjs.MovieClip) {
			copyobj = createClone(obj);
		} else {
			copyobj = obj.clone(true);

		}
		newMC.addChild(copyobj);
		copyobj.regX = obj.regX;
		copyobj.regY = obj.regY;
		copyobj.x = 0;
		copyobj.y = 0;
	}
	newMC.regX = mc.regX;
	newMC.regY = mc.regY;
	newMC.name = mc.name;
	newMC.mouseEnabled = false;
	return newMC;
}

function establishSessionHighscore() {
	var isInternetExp8 = window.XDomainRequest ? true : false;
	var xmlHtpRequestHolder = createCrossDomainRequest();
	var sessionConnectURL = gamevars(0)["session"] + getTimer();
	function createCrossDomainRequest(sessionConnectURL, onSessionEstablished) {
		var sessionRequest;
		if (isInternetExp8) {
			sessionRequest = new windowClassHolder["XDomainRequest"]();
		} else {
			sessionRequest = new XMLHttpRequest();
		}
		return sessionRequest;
	}
	callOtherDomain();
	function callOtherDomain() {
		if (xmlHtpRequestHolder) {
			if (isInternetExp8) {
				xmlHtpRequestHolder["onload"] = sessionResponseHandleer;
				xmlHtpRequestHolder["open"]("POST", sessionConnectURL, true);
				xmlHtpRequestHolder["setRequestHeader"]("Content-Type", "application/x-www-form-urlencoded");
				xmlHtpRequestHolder["send"]("id=00185708");
			} else {
				xmlHtpRequestHolder["open"]('POST', sessionConnectURL, true);
				xmlHtpRequestHolder["setRequestHeader"]("Content-Type", "application/x-www-form-urlencoded");
				xmlHtpRequestHolder["onreadystatechange"] = onSessionEstablished;
				xmlHtpRequestHolder["send"]("id=00185708");
			}
		} else {
			gamevars(0)["hsapi_status"] = "no";
		}
	}
	function onSessionEstablished(evtXHR) {
		if (xmlHtpRequestHolder["readyState"] == 4) {
			if (xmlHtpRequestHolder["status"] == 200) {
				sessionResponseHandleer();
			} else {
				gamevars(0)["hsapi_status"] = "no";
			}
		}
	}
	function sessionResponseHandleer() {
		var objxx = {
			n: "game_name",
			u: "game_url",
			w: "game_width",
			h: "game_height",
			sid: "session_id",
			status: "hsapi_status"
		};
		var response = xmlHtpRequestHolder.responseText;
		var obj = JSON.parse('{"' + decodeURI(response).replace(/"/g, '\\"').replace(/&/g, '","').replace(/=/g, '":"') + '"}');
		for (var sk in obj) {
			gamevars(0)[objxx[sk]] = obj[sk];
		}
	}
}

function getRotatedRect(angle, width, height) {
	var angle = angle * Math.PI / 180,
		sin = Math.sin(angle),
		cos = Math.cos(angle);
	var x1 = cos * width,
		y1 = sin * width;
	var x2 = -sin * height,
		y2 = cos * height;
	var x3 = cos * width - sin * height,
		y3 = sin * width + cos * height;
	var minX = Math.min(0, x1, x2, x3),
		maxX = Math.max(0, x1, x2, x3),
		minY = Math.min(0, y1, y2, y3),
		maxY = Math.max(0, y1, y2, y3);
	var rotatedWidth = maxX - minX,
		rotatedHeight = maxY - minY;
	return new createjs.Rectangle(minX, minY, rotatedWidth, rotatedHeight);
}

function initBoard(sBoardMC) {
	sBoardMC.stop();
	try{
		sBoardMC.bg = new lib.Background();
	}catch(e){
		sBoardMC.bg = new (gamevars(0).lib.Background)();
	}
	gamevars(3, sBoardMC.bg);
	sBoardMC.commonItems.closeBtn.scaleX = sBoardMC.commonItems.closeBtn.scaleY = 2;
	var rec = sBoardMC.bg.getBounds();
	sBoardMC.bg.scaleX = gamevars(0).stw /rec.width;
	sBoardMC.bg.scaleY = gamevars(0).sth /rec.height;
	gamevars(3, sBoardMC);	
	var stw = gamevars(0).stw-30;
	var sth = gamevars(0).sth-30;
	var tw = 665;
	var th = 720;
	
	if(tw > stw){
		sBoardMC.scaleX = sBoardMC.scaleY = (stw/tw);
		tw *= sBoardMC.scaleX;
		th *= sBoardMC.scaleY;
	}
	if(th > sth){
		sBoardMC.scaleX = sBoardMC.scaleY = (sth/th);
		tw *= sBoardMC.scaleX;
		th *= sBoardMC.scaleY;
	}
	
	var rect = sBoardMC.getTransformedBounds();
	sBoardMC.x = (stw+30)/2;
	sBoardMC.y = (Math.abs((sth+30)-rect.height)/2)+15;
	var game_scr = gamevars(0).totalScore();
	sBoardMC.alpha = 1;
	gamevars(6, [onmouseb]);
	sBoardMC.diff = 0;
	var inputText = sBoardMC.HE.pnTxt;
	scoreText = sBoardMC.HE.txtscore;
	var keypad = sBoardMC.HE.keypad;
	if(sth>stw){
		keypad.y += 220;
	}
	keypad.visible = false;
	sBoardMC.caplet = true;
	scoreText.text = "" + game_scr;
	sBoardMC.nameStr = gamevars(0).sobj["playername"];
	if (sBoardMC.nameStr == undefined || sBoardMC.nameStr == null || sBoardMC.nameStr.length>3) {
		sBoardMC.nameStr = "Enter your name here";
		sBoardMC.caplet = true;
	} else {
		sBoardMC.caplet = false;
		
	}
	inputText.text = sBoardMC.nameStr;
	sBoardMC.scrollBar = sBoardMC.HV.scrollBar;
	sBoardMC.commonItems.gamename.text = gamevars(0, null)["gamename"].toUpperCase();
	function updateScroll() {
		var ran = (sBoardMC.scrollBar.mover.y + 144) / 288;
		var curY = -20 - (sBoardMC.diff * ran);
		sBoardMC.HV.scoreMc.y = curY;
	}
	function setNameStr() {
		if (sBoardMC.nameStr == "Enter your name here" || sBoardMC.nameStr == "" || sBoardMC.nameStr == " ") {
			sBoardMC.nameStr = "";
			inputText.text = "";
		} else {
			inputText.text = sBoardMC.nameStr;
		}
	}
	function onmouseb(eobj) {
		var targetName = eobj.target.name;
		switch (eobj.target.getName()) {
			case "subBtn":
				if (sBoardMC.nameStr == "Enter your name here") {
					sBoardMC.HE.errort.text = "Please enter name  for submitting score.";
					return;
				}
				if (sBoardMC.nameStr.length < 3) {
					sBoardMC.HE.errort.text = "Please enter name with at least 4 characters";
				} else {
					keypad.visible = false;
					gamevars(0).sobj["playername"] = sBoardMC.nameStr;
					gamevars(0).saveGame();
					sBoardMC.addChild(sBoardMC.HL);
					sBoardMC.HL.msgTxt.text = "Dear " + sBoardMC.nameStr + " your score: " + game_scr + " is being submitted";
					sBoardMC.addChild(sBoardMC.commonItems);
					callPHP("id=f99233b4&player=" + sBoardMC.nameStr + "&score=" + game_scr + "&plrid=" + gamevars(0).sobj["playerid"] + "&type=max&sid=&mode=" + gamevars(0)["gamename"]);
				}
				break;
			case "upBtn":
				sBoardMC.scrollBar.mover.y -= 20;
				if (sBoardMC.scrollBar.mover.y <= -144) {
					sBoardMC.scrollBar.mover.y = -144;
				}
				updateScroll();
				break;
			case "downBtn":
				sBoardMC.scrollBar.mover.y += 20;
				if (sBoardMC.scrollBar.mover.y >= 144) {
					sBoardMC.scrollBar.mover.y = 144;
				}
				updateScroll();
				break;
			case "closeBtn":
			case "closeBtn":
				gamevars(4, sBoardMC);
				gamevars(7, [onmouseb]);
				gamevars(4, sBoardMC.bg);
				break;
			case "pnTxt":
			case "dokrirosabu":
				if (sBoardMC.nameStr == "Enter your name here") {
					sBoardMC.caplet = true;
				}
				sBoardMC.HE.addChild(keypad);
				keypad.visible = true;
				sBoardMC.HE.errort.text = "";
				setNameStr();
				
				break;
			case "spaceBtn":
				sBoardMC.nameStr += " ";
				inputText.text = sBoardMC.nameStr;
				sBoardMC.caplet = true;
				break;
			case "backSpaceBtn":
				sBoardMC.nameStr = sBoardMC.nameStr.substr(0, sBoardMC.nameStr.length - 1);
				inputText.text = sBoardMC.nameStr;
				if (sBoardMC.nameStr == "") {
					sBoardMC.caplet = true;
				}
				break;
			case "clearInputText":
				sBoardMC.caplet = true;
				inputText.text = "Enter your name here";
				sBoardMC.nameStr = ""
				break;
		}
		if (targetName && targetName.length == 1) {
			if (sBoardMC.nameStr.length < 3) {
				sBoardMC.nameStr += (sBoardMC.caplet) ? targetName : targetName.toLowerCase();
				inputText.text = sBoardMC.nameStr;
				sBoardMC.caplet = false;
			} else {
				sBoardMC.HE.errort.text = "You can enter only 3 letters.";
			}
			return;
		}

	}

	function callPHP(params) {
		var httpc = new XMLHttpRequest(); // simplified for clarity
		var url = gamevars(0)["hsSubmit"];
		httpc.open("POST", url, true); // sending as POST
		httpc.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		httpc.onreadystatechange = function () { //Call a function when the state changes.
			if (httpc.readyState == 4 && httpc.status == 200) { // complete and no errors
				var str = (httpc.responseText);
				var obj = JSON.parse('{"' + decodeURI(str).replace(/"/g, '\\"').replace(/&/g, '","').replace(/=/g, '":"') + '"}');
				if (gamevars(0).sobj["playerid"] != obj.plrid) {
					gamevars(0).sobj["playerid"] = obj.plrid;
					gamevars(0).saveGame();
				}
				switch (obj.status) {
					case "submitted":
						sBoardMC.HL.msgTxt.text = "Dear " + sBoardMC.nameStr + " your score: " + game_scr + " is submitted successfully\nLoading Highscore table...";
						break;
					case "updated":
						sBoardMC.HL.msgTxt.text = "Dear " + sBoardMC.nameStr + " your score: " + game_scr + " is updated in highscore table\nLoading Highscore table...";
						break;
					case "nochange":
						sBoardMC.HL.msgTxt.text = "Dear " + sBoardMC.nameStr + " your score: " + game_scr + " is less than  or equal to previous\nLoading Highscore table...";
						break;
				}

				var oReq = new XMLHttpRequest();
				oReq.onload = function (e) {
					var respText = oReq.response;
					var parser = new DOMParser();
					var xmlDoc = parser.parseFromString(respText, "text/xml");
					sBoardMC.addChild(sBoardMC.HV);
					sBoardMC.addChild(sBoardMC.commonItems);
					sBoardMC.HV.yscr.text = sBoardMC.nameStr + ": " + game_scr;
					var ctrn = sBoardMC.HV.scoreMc;
					while (ctrn.numChildren) {
						ctrn.removeChildAt(0);
					}
					var len = xmlDoc.getElementsByTagName("name").length;
					for (i = 0; i < len; i++) {
						var scoreEntry;
						try{
						scoreEntry = new lib.ScoreEntery();
						}catch(e){
							scoreEntry = new (gamevars(0).lib.ScoreEntery)();
						}
						scoreEntry.nameTxt.text = xmlDoc.getElementsByTagName("name")[i].childNodes[0].nodeValue;
						scoreEntry.rankTxt.text = xmlDoc.getElementsByTagName("rank")[i].childNodes[0].nodeValue;
						scoreEntry.scoreTxt.text = xmlDoc.getElementsByTagName("score")[i].childNodes[0].nodeValue;
						ctrn.addChild(scoreEntry);
						scoreEntry.y = i * 33;
					}
					if (len > 10) {
						sBoardMC.diff = (len * 33) - 330;
						var o = new createjs.Shape(new createjs.Graphics().f("#000").dr(-190, -120, 388, 330));
						ctrn.mask = o;
						sBoardMC.scrollBar.visible = true;
						sBoardMC.scrollBar.mover.on("pressmove", function (evt) {
							var point = sBoardMC.scrollBar.globalToLocal(evt.stageX, evt.stageY);
							evt.target.y = point.y;
							if (evt.target.y > 144) {
								evt.target.y = 144;
							}
							if (evt.target.y < -144) {
								evt.target.y = -144;
							}
							updateScroll();

						});
					} else {
						sBoardMC.scrollBar.visible = false;
					}
				}				
				oReq.open("GET", gamevars(0)["hsGet"] + "?id=f99233b4&mode=" + gamevars(0)["gamename"]);
				oReq.responseType = "text";
				oReq.send();
			}
		}
		httpc.send(params);
	}
}

function getParameterByName(name) {
    var  url = window.location.href;
    name = name.replace(/[\[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
}

function getRandomColor(type = 1) {
	var letters = (type == 1 ? 'bcdef'.split('') : '12345'.split(''));
	var color = '#';
	for (var i = 0; i < 6; i++) {
		color += letters[Math.floor(Math.random() * letters.length)];
	}
	return color;
}

/////////////////  partical.js code //////////////////////////
var ps = ps || {};

ps.removePartical = function (particalMc) {
	particalMc.parent.removeChild(particalMc);
}
ps.getParticleObject = function (PartClass, ParentMc, time, distance, pos) {
	var particalContainer = new createjs.Container();
	ParentMc.addChild(particalContainer);
	var particalMc = new PartClass();
	particalContainer.addChild(particalMc);
	particalContainer.x = pos.x;
	particalContainer.y = pos.y;
	particalContainer.partical = particalMc;
	return particalContainer;
}
ps.setLinearAnimation = function (targetMc, time, distance) {
	
	createjs.Tween.get(targetMc.partical, {
		useTicks: true
	}).to({
		x: distance,
		alpha: 0,
		scaleX: 0,
		scaleY: 0
	}, time).call(ps.removePartical, [targetMc]);
}
ps.createRoundAnimation = function (PartClass, ParentMc, pos, time = 10, distance = 100, numLeaf = 10) {
	for (var i = 0; i < numLeaf; i++) {
		var particalContainer = ps.getParticleObject(PartClass, ParentMc, time, distance, pos);
		particalContainer.rotation = i * (360 / numLeaf);
		ps.setLinearAnimation(particalContainer, time, distance);
	}
}

ps.createArrowFlash = function (PartClass, ParentMc, pos, time = 10, distance = 100, numLeaf = 10, directionRange) {
	var dr = directionRange || {
		min: -30,
		add: 60
	}
	
	for (var i = 0; i < numLeaf; i++) {
		var particalContainer = ps.getParticleObject(PartClass, ParentMc, time, distance, pos);
		particalContainer.rotation = dr.min + Math.random() * dr.add;
		ps.setLinearAnimation(particalContainer, time, distance);
	}
}

ps.createFountainEffect =  function(PartClass, ParentMC, pos, configObject) {
	var configObj = configObject || {
		mvx: -10,
		avx: 20,
		mvy: -18,
		avy: 5,
		numParts: 10,
		gravity: 1,
		rotation: 0,
		defuseVy: 13
	};
	var particalContainer = new createjs.Container();
	var particalArray = [];
	particalContainer.mouseChildren = false;
	particalContainer.mouseEnabled = false;
	particalContainer.x = pos.x;
	particalContainer.y = pos.y;
	ParentMC.addChild(particalContainer);
	particalContainer.rotation = configObj.rotation;
	for (var i = 0; i < configObj.numParts; i++) {
		var partical = new PartClass();
		particalContainer.addChild(partical);
		particalArray.push(partical);
		partical.vx = configObj.mvx + Math.random() * configObj.avx;
		partical.vy = configObj.mvy + Math.random() * configObj.avy;
		partical.gravity = configObj.gravity;
	}
	gamevars(28, [moveTheParticals]);
	function moveTheParticals(evt) {
		for (var i = particalArray.length - 1; i >= 0; i--) {
			var partical = particalArray[i];
			partical.x += partical.vx;
			partical.y += partical.vy;
			partical.vy += partical.gravity;
			if (partical.vy > configObj.defuseVy) {
				partical.alpha -= 0.1;
			}
			if (partical.alpha == 0) {
				particalContainer.removeChild(partical);
				particalArray.splice(particalArray.indexOf(partical), 1);
			}
		}
		if (particalArray.length == 0) {
			gamevars(29, [moveTheParticals]);
			particalContainer.parent.removeChild(particalContainer);
		}
	}
}


var isMobile = {
    Android: function() {
        return navigator.userAgent.match(/Android/i);
    },
    BlackBerry: function() {
        return navigator.userAgent.match(/BlackBerry/i);
    },
    iOS: function() {
        return navigator.userAgent.match(/iPhone|iPad|iPod/i);
    },
    Opera: function() {
        return navigator.userAgent.match(/Opera Mini/i);
    },
    Windows: function() {
        return navigator.userAgent.match(/IEMobile/i);
    },
    any: function() {
        return (isMobile.Android() || isMobile.BlackBerry() || isMobile.iOS() || isMobile.Opera() || isMobile.Windows());
    }
};